// Course.java
//
// Ethan Bolker
// May, 2004 for cs110 final project

import java.util.*;

/**
 * A Course object in the WISE system.
 */
public class Course extends WISEObject
{
    // private fields for Course attributes
    private TimeOfDay timeOfDay;
    private int capacity;
    private Professor professor; // Professor teaching this Course

    /**
     * Construct a new Course.
     */
    public Course( StringTokenizer st )
	throws WISEException
    {
	super(st.nextToken());
	String days = st.nextToken();
	String time = st.nextToken();
	timeOfDay   = new TimeOfDay(days, time);
	capacity    = Integer.parseInt(st.nextToken());
	setList( new ClassList(this) );
    }

    public String toString()
    {
	return getName() + "\t" + timeOfDay + "\t" + capacity;
    }

    /**
     * Add a Student to this Course.
     */
    public void add( Student student )
	throws WISEException
    {
	if (getList().size() >= capacity) {
	    throw new WISEException( "Course full: " + getName());
	}
	getList().put( student );
    }

    /** 
     * Does this course meet at the specified time, and
     * is there room in it?
     */
    public boolean isAvailable( TimeOfDay tod )
    {
	return ( tod.equals(timeOfDay) && 
		(getList().size() < capacity ));
    }

    /**
     * Assign a Professor to teach this Course.
     */
    public void add( Professor professor )
	throws WISEException
    {
	Professor oldProf = this.professor;
	if ( professor == oldProf ) {
	    return;
	}
	professor.add(this); 
	this.professor = professor;
	if ( oldProf != null ) {
	    oldProf.unAssign( this );
	}
    }

    public TimeOfDay getTimeOfDay()
    {
	return timeOfDay;
    }

    private class ClassList extends StudentList
    {
	public ClassList( Course course )
	{
	    super("Class list for " + course);
	}

	public String getHeader( ) 
	{
	    StringBuffer buf = new StringBuffer("*** CS110 WISE ***\n");
	    buf.append( this.getName());
	    buf.append( "\n");
	    buf.append( (new Date()).toString() );
	    buf.append( "\n\nProfessor " + 
			(professor == null ?
			 "**unassigned**"  :
			 professor.getName()));
	    buf.append( "\nList of students\nname  id");
	    return buf.toString();
	}
    }
}
