// Professor.java
//
// Ethan Bolker
// May, 2004 for cs110 final project

import java.util.*;

/**
 * A Professor object in the WISE system.
 */
public class Professor extends WISEObject
{
    // private fields for Professor attributes
    private int teachingLoad;

    /**
     * Construct a new Professor.
     */
    public Professor( StringTokenizer st )
    {
	super(st.nextToken());
	teachingLoad = Integer.parseInt(st.nextToken());
	setList( new Schedule(this) );
    }

    public String toString()
    {
	return getName() + "\t" + teachingLoad;
    }

    /**
     * Can this Professor teach at this time?
     */
    public boolean isAvailable( TimeOfDay tod )
    {
	return ( super.isAvailable(tod) && 
		(getList().size() < teachingLoad));
    }

    public void unAssign( Course course ) 
    {
	getList().remove( course );
    }

    /**
     * Add a Course for this Professor.
     */
    public void add( Course course )
	throws WISEException
    {
	if (getList().contains( course )) {
	    throw new WISEException( 
 	      getName() +  " already assigned to course " +
	      course.getName());
	}
	if ( ! isAvailable( course.getTimeOfDay())) {
	    throw new WISEException( "Professor " +
 	      getName() +  " not free " +  course.getTimeOfDay());
	}
	if (getList().size() >= teachingLoad) {
	    throw new WISEException( "Schedule full: " + getName());
	}
	getList().put( course );
    }

    private class Schedule extends CourseList
    {
	public Schedule( Professor professor )
	{
	    super("Schedule for Professor " + professor.getName());
	}

	// same code in Student.java!
	public String getHeader( ) 
	{
	    StringBuffer buf = new StringBuffer("*** CS110 WISE ***\n");
	    buf.append( this.getName());
	    buf.append( "\n");
	    buf.append( (new Date()).toString() );
	    buf.append( "\n\nCourses\nname  time  capacity");
	    return buf.toString();
	}
    }
}
