// WISE.java
//
// For final project, cs110. 
// Based on example in exam2, with ideas from Juno
// Ethan Bolker  Spring 2004

import java.util.*;

public class WISE
{
    // dispatch table for lists of students, courses, professors
    private TreeMap allLists = new TreeMap();

    // dispatch table for WISE commands
    private TreeMap commands = new TreeMap();

    private Terminal t;

    /**
     * Build a WISE instance.
     */
    public WISE( String[] args ) 
    {
	// parse command line arguments
	String courseFile    = null;
	String studentFile   = null;
	String professorFile = null;
	try {
	    int i = 0;
	    boolean echo = false;
	    if ( args[0].equals("-e" ) ) {
		echo = true;
		i = 1;
	    }
	    courseFile    = args[i++];
	    studentFile   = args[i++];
	    professorFile = args[i];
	    t = new Terminal( echo );
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    System.out.println
		("usage: java WISE [-e] courses students professors");
	    System.exit(0);
	}

	// build the object lists dispatch table
	try {
	    allLists.put("courses",
			 new CourseList("courses", courseFile));
	    allLists.put("students",
			 new StudentList("students", studentFile));
	    allLists.put("professors", 
			 new ProfessorList("professors", professorFile));
	}
	catch (WISEException e) {
	    t.errPrintln(e);
	    System.exit(0);
	}
	
	// build the command dispatch table
	commands.put( "type",      new TypeCommand() );
	commands.put( "enroll",    new EnrollCommand() );
	commands.put( "assign",    new AssignCommand() );
	commands.put( "file",      new FileCommand() );
	commands.put( "available", new AvailableCommand() );

	// this.test();  // no longer needed
    }

    /**
     * Start the WISE system:
     * do until exit
     *   get command and execute it
     */
    public void start() 
    {
	t.println("Welcome to Ethan Bolker's WISE implementation.");
	while( true ) {
	    String inputLine = t.readLine("WISE> ");
	    StringTokenizer st = stripComments(inputLine);
	    if (st.countTokens() == 0) {           // skip blank line
		continue;
	    }
	    String commandName = st.nextToken();
	    if (commandName.equals("exit")) {
		break;
	    }
	    try { 
		WISECommand commandObject = 
		   (WISECommand)commands.get( commandName );
		if (commandObject == null ) {
		    throw new WISEException
			("Unknown command: " + commandName );
		}           
 		commandObject.doIt( st, this );
	    }                                                
	    catch (WISEException e) {                               
		t.errPrintln( e.getMessage() );               
	    }                                                       
	    catch (Exception e) {                                   
		t.errPrintln( "unexpected WISE error.\n " +
			      "Please call tech support.");  
		t.errPrintln( e.toString() );                 
	    }                                                       
	}
    }

    // Strip characters from '#' to end of line, create and
    // return a StringTokenizer for what's left.

    private StringTokenizer stripComments( String line )
    {
        int commentIndex = line.indexOf('#');
        if (commentIndex >= 0) {
            line = line.substring(0,commentIndex);
        }
        return new StringTokenizer(line);
    }

    // *********** List retrieval tools **********

    /** 
     * Retrieve one of the lists, given its name.
     */
    public WISEList get( String listName )
	throws WISEException
    {
	if ( !allLists.containsKey(listName)) {
	    throw new WISEException("No such list: " + listName);
	}
	return (WISEList)allLists.get(listName);
    }

    // Get Student, Course and Professor objects, by name.
    public Student getStudent( String name )
	throws WISEException
    {
	return (Student)(get("students").get(name));
    }

    public Course getCourse( String name )
	throws WISEException
    {
	return (Course)(get("courses").get(name));
    }

    public Professor getProfessor( String name )
	throws WISEException
    {
	return (Professor)(get("professors").get(name));
    }

    // routine getter for Terminal
    public Terminal getTerminal() 
    {
	return t;
    }

    // test new stuff here - comment out call when type
    // command has been implemented
    private void test()
    {
	t.println("--------- BEGIN TESTING ----------");
	t.println("test showing that input files have been read");
	try {
	    WISEList list;
	    list = get("courses"); 
	    t.println( list.getPrintableList());
	    list = get("students"); 
	    t.println( list.getPrintableList());
	    list = get("professors"); 
	    t.println( list.getPrintableList());
	}
	catch( WISEException e ) {
	    System.err.println(e.getMessage());
	}	
    	System.out.println("--------- END TESTING ----------\n");
    }

    /**
     * Create WISE, then open it for business.
     */
    public static void main( String[] args )
    {
	WISE system = new WISE(args);
	system.start();
    }
}
