// WISEList.java
//
// For final project, cs110. 
// Based on example in exam2
// Ethan Bolker  Spring 2004

import java.util.*;
import java.io.*;

/**
 * This abstract class wraps a TreeMap, providing Map
 * functionality for WISE maps.
 */
public abstract class WISEList
{
    private String name;
    private Map elements = new TreeMap();

    /**
     * Create an empty WISEList.
     */
    public WISEList( String name )
    {
	this.name = name;
    }
    
    /**
     * Create a WISEList; populate it by reading a file.
     *
     * @param fileName the file to read.
     */
    public WISEList( String name, String fileName )
	throws WISEException
    {
	this(name);
        BufferedReader inStream  = null;
        try {
            // open the file
            inStream =  new BufferedReader(new FileReader(fileName));

            // parse a line at a time
	    String line;
            while ((line = inStream.readLine()) != null) {
		line = line.trim();
		if (( line.length() == 0 ) || line.startsWith("#") ) {
		    continue;
		}
		WISEObject obj = parseLine( new StringTokenizer(line) ); 
		this.put( obj );
            }
        }
        catch (Exception e) {
	    throw new WISEException( e.toString() );
        }
        finally { // close the file
            try {
		inStream.close();
            }
            catch (Exception e) {
		throw new WISEException( e.toString() );
            }
        }
    }

    /**
     * Abstract parseLine method is written in each subclass, 
     * since Student, Course and Professor lines are different.
     */
    public abstract WISEObject parseLine( StringTokenizer st )
	throws WISEException;

    // Standard Map management tools
    public void put( WISEObject obj ) 
    {
	elements.put(obj.getName(), obj);
    }

    public void remove( WISEObject obj )
    {
	elements.remove(obj.getName());
    }

    public Iterator iterator( ) 
    {
	return elements.values().iterator();
    }

    public boolean contains( WISEObject obj )
    {
	return elements.containsValue(obj);
    }

    public int size()
    {
	return elements.size();
    }

    /**
     * Look up a WISEObject on this WISEList
     */
    public WISEObject get( String name )
	throws WISEException
    {
	WISEObject obj = (WISEObject)elements.get(name);
	if (obj == null) {
	    throw new WISEException( name + " not found in  " + this.name);
	}
	return obj;
    }

    /**
     * Each subclass decides what its header should look like
     * when list is printed.
     */
    public abstract String getHeader( );

    public String getName()
    {
	return name;
    }
   
    public String getPrintableList( )
    {
	// should do this with a StringBuffer
	String s = getHeader() + '\n';
	for (Iterator i = elements.values().iterator(); i.hasNext(); ) {
	    s += i.next().toString() + '\n';
	}
	return s;
    }
}
